@@ -35,15 +35,16 @@ static inline void _spa_autoptr_cleanup_func_ ## name (__typeof__(type) **thing)
 
 #define spa_exchange(var, new_value) \
 __extension__ ({ \
-	__typeof__(var) _old_value = (var); \
-	(var) = (new_value); \
+	__typeof__(var) *_ptr = &(var); \
+	__typeof__(var) _old_value = *_ptr; \
+	*_ptr = (new_value); \
 	_old_value; \
 })
 
-#if __GNUC__ > 10 || defined(__clang__)
+#if __GNUC__ >= 10 || defined(__clang__)
 #define spa_steal_ptr(ptr) ((__typeof__(*(ptr)) *) spa_exchange((ptr), NULL))
 #else
-#define spa_steal_ptr(ptr) ((__typeof__(ptr)) spa_exchange((ptr), NULL))
+#define spa_steal_ptr(ptr) spa_exchange((ptr), NULL)
 #endif
 
 #define spa_steal_fd(fd) spa_exchange((fd), -1)
@@ -52,16 +53,6 @@ __extension__ ({ \
 
 #include <stdlib.h>
 
-
-#if __GNUC__ > 10 || defined(__clang__)
-#define spa_clear_ptr(ptr, destructor) \
-__extension__ ({ \
-	__typeof__(*(ptr)) *_old_value = spa_steal_ptr(ptr); \
-	if (_old_value) \
-		destructor(_old_value); \
-	(void) 0; \
-})
-#else
 #define spa_clear_ptr(ptr, destructor) \
 __extension__ ({ \
 	__typeof__(ptr) _old_value = spa_steal_ptr(ptr); \
@@ -69,7 +60,6 @@ __extension__ ({ \
 		destructor(_old_value); \
 	(void) 0; \
 })
-#endif
 
 static inline void _spa_autofree_cleanup_func(void *p)
 {
