@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2021 Wim Taymans <wim.taymans@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2021 Wim Taymans <wim.taymans@gmail.com> */
+/* SPDX-License-Identifier: MIT */
 
 #include <spa/param/audio/format-utils.h>
 #include <spa/utils/hook.h>
@@ -129,19 +109,6 @@ static const struct spa_dict_item module_remap_source_info[] = {
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
-static void position_to_props(struct spa_audio_info_raw *info, struct pw_properties *props)
-{
-	char *s, *p;
-	uint32_t i;
-
-	pw_properties_setf(props, SPA_KEY_AUDIO_CHANNELS, "%u", info->channels);
-	p = s = alloca(info->channels * 8);
-	for (i = 0; i < info->channels; i++)
-		p += spa_scnprintf(p, 8, "%s%s", i == 0 ? "" : ",",
-				channel_id2name(info->position[i]));
-	pw_properties_set(props, SPA_KEY_AUDIO_POSITION, s);
-}
-
 static int module_remap_source_prepare(struct module * const module)
 {
 	struct module_remap_source_data * const d = module->user_data;
@@ -207,26 +174,19 @@ static int module_remap_source_prepare(struct module * const module)
 		pw_properties_set(props, "master", NULL);
 	}
 
-	if (module_args_to_audioinfo(module->impl, props, &playback_info) < 0) {
+	if (module_args_to_audioinfo_keys(module->impl, props,
+			NULL, NULL, "channels", "channel_map", &playback_info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
 	capture_info = playback_info;
-
-	if ((str = pw_properties_get(props, "master_channel_map")) != NULL) {
-		struct channel_map map;
-
-		channel_map_parse(str, &map);
-		if (map.channels == 0 || map.channels > SPA_AUDIO_MAX_CHANNELS) {
-			pw_log_error("invalid channel_map '%s'", str);
-			res = -EINVAL;
-			goto out;
-		}
-		channel_map_to_positions(&map, capture_info.position);
-		pw_properties_set(props, "master_channel_map", NULL);
+	if (module_args_to_audioinfo_keys(module->impl, props,
+			NULL, NULL, NULL, "master_channel_map", &capture_info) < 0) {
+		res = -EINVAL;
+		goto out;
 	}
-	position_to_props(&playback_info, playback_props);
-	position_to_props(&capture_info, capture_props);
+	audioinfo_to_properties(&playback_info, playback_props);
+	audioinfo_to_properties(&capture_info, capture_props);
 
 	if ((str = pw_properties_get(props, "remix")) != NULL) {
 		/* Note that the boolean is inverted */
@@ -246,7 +206,6 @@ static int module_remap_source_prepare(struct module * const module)
 out:
 	pw_properties_free(playback_props);
 	pw_properties_free(capture_props);
-
 	return res;
 }
 
