@@ -35,6 +35,9 @@
 #if defined(__FreeBSD__) || defined(__MidnightBSD__)
 #include <sys/thr.h>
 #endif
+#if defined(__GNU__)
+#include <mach.h>
+#endif
 #include <fcntl.h>
 #include <unistd.h>
 #include <pthread.h>
@@ -221,6 +224,9 @@ static pid_t _gettid(void)
 	long pid;
 	thr_self(&pid);
 	return (pid_t)pid;
+#elif defined(__GNU__)
+       mach_port_t thread = mach_thread_self();
+       return (pid_t)thread;
 #else
 #error "No gettid impl"
 #endif
@@ -923,14 +929,11 @@ static int check_rtkit(struct impl *impl, struct pw_context *context, bool *can_
 	return 0;
 }
 
-static int do_rtkit_setup(struct spa_loop *loop, bool async, uint32_t seq,
-		const void *data, size_t size, void *user_data)
+static int rtkit_get_bus(struct impl *impl)
 {
-	struct impl *impl = user_data;
 	int res;
-	long long retval;
 
-	pw_log_debug("enter rtkit setup");
+	pw_log_debug("enter rtkit get bus");
 
 	/* Checking xdg-desktop-portal. It works fine in all situations. */
 	if (impl->rtportal_enabled)
@@ -967,6 +970,18 @@ static int do_rtkit_setup(struct spa_loop *loop, bool async, uint32_t seq,
 			return res;
 		}
 	}
+
+	return 0;
+}
+
+static int do_rtkit_setup(struct spa_loop *loop, bool async, uint32_t seq,
+		const void *data, size_t size, void *user_data)
+{
+	struct impl *impl = user_data;
+	long long retval;
+
+	pw_log_debug("enter rtkit setup");
+
 	/* get some properties */
 	if (rtkit_get_int_property(impl, "MaxRealtimePriority", &retval) < 0) {
 		retval = 1;
@@ -1076,7 +1091,14 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 #ifdef HAVE_DBUS
 	impl->use_rtkit = use_rtkit;
 	if (impl->use_rtkit) {
-		impl->thread_loop = pw_thread_loop_new("module-rt", NULL);
+		struct spa_dict_item items[] = {
+			{ "thread-loop.start-signal", "true" }
+		};
+		if ((res = rtkit_get_bus(impl)) < 0)
+			goto error;
+
+		impl->thread_loop = pw_thread_loop_new("module-rt",
+			&SPA_DICT_INIT_ARRAY(items));
 		if (impl->thread_loop == NULL) {
 			res = -errno;
 			goto error;
